@node RPC Function Reference
@section RPC Function Reference
@cindex RPC function reference
@cindex Function reference, RPC

These routines allow C programs to make procedure calls on other
machines across the network.  First, the client calls a procedure to
send a data packet to the server.  Upon receipt of the packet, the
server calls a dispatch routine to perform the requested service, and
then sends back a reply.  Finally, the procedure call returns to the
client.

Routines that are used for Secure RPC (DES authentication) are
not available in this release.

@c described in @cite{rpc_secure}(3N).  Secure RPC can be used only if DES
@c encryption is available.

@menu
* auth_destroy()::
* authnone_create()::
* authunix_create()::
* authunix_create_default()::
* callrpc()::
* clnt_broadcast()::
* clnt_call()::
* clnt_destroy()::
* clnt_create()::
* clnt_control()::
* clnt_freeres()::
* clnt_geterr()::
* clnt_pcreateerror()::
* clnt_perrno()::
* clnt_perror()::
* clnt_spcreateerror()::
* clnt_sperrno()::
* clnt_sperror()::
* clntraw_create()::
* clnttcp_create()::
* clntudp_create()::
* clntudp_bufcreate()::
* get_myaddress()::
* pmap_getmaps()::
* pmap_getport()::
* pmap_rmtcall()::
* pmap_set()::
* pmap_unset()::
* registerrpc()::
* rpc_createerr::
* svc_destroy()::
* svc_fdset::
* svc_fds::
* svc_freeargs()::
* svc_getargs()::
* svc_getcaller()::
* svc_getreqset()::
* svc_getreq()::
* svc_register()::
* svc_run()::
* svc_sendreply()::
* svc_unregister()::
* svcerr_auth()::
* svcerr_decode()::
* svcerr_noproc()::
* svcerr_noprog()::
* svcerr_progvers()::
* svcerr_systemerr()::
* svcerr_weakauth()::
* svcraw_create()::
* svctcp_create()::
* svcfd_create()::
* svcudp_bufcreate()::
* svcudp_create()::
* xdr_accepted_reply()::
* xdr_authunix_parms()::
* xdr_callhdr()::
* xdr_callmsg()::
* xdr_opaque_auth()::
* xdr_pmap()::
* xdr_pmaplist()::
* xdr_rejected_reply()::
* xdr_replymsg()::
* xprt_register()::
* xprt_unregister()::
@end menu

@need 1600
@node auth_destroy()
@subsection auth_destroy()
@findex auth_destroy()

@example
void
auth_destroy(auth)
AUTH *auth;
@end example

@noindent A macro that destroys the authentication information associated with
@code{auth}.  Destruction usually involves deallocation of private data
structures. The use of @code{auth} is undefined after calling
@code{auth_destroy()}.

@need 1200
@node authnone_create()
@subsection authnone_create()
@findex authnone_create()

@example
AUTH *
authnone_create()
@end example

@noindent Create and returns an @code{RPC} authentication handle 
that passes nonusable authentication information with each remote
procedure call. This is the default authentication used by @code{RPC.}

@need 2000
@node authunix_create()
@subsection authunix_create()
@findex authunix_create()

@example
AUTH *
authunix_create(host, uid, gid, len, aup_gids)
char *host;
int uid, gid, len, *aup_gids;
@end example

@noindent Create and return an @code{RPC} authentication handle
 that contains Unix authentication information.  The parameter
@code{host} is the name of the machine on which the information was
created; @code{uid} is the user's user @code{ID}; @code{gid} is the
user's current group @code{ID}; @code{len} and @code{aup_gids} refer to
a counted array of groups to which the user belongs.  It is easy to
impersonate a user.

@need 1000
@node authunix_create_default()
@subsection authunix_create_default()
@findex authunix_create_default()

@example
AUTH *
authunix_create_default()
@end example

@noindent Calls
@ifinfo
@ref{authunix_create()}
@end ifinfo
@iftex
@code{authunix_create()}
@end iftex
with the appropriate parameters.

@need 2600
@node callrpc()
@subsection callrpc()
@findex callrpc()

@example
int
callrpc(host, prognum, versnum, procnum, inproc, in, outproc, out)
char *host;
u_long prognum, versnum, procnum;
char *in, *out;
xdrproc_t inproc, outproc;
@end example

@noindent Call the remote procedure associated with @code{prognum},
@code{versnum}, and @code{procnum} on the machine, @code{host}.  The
parameter @code{in} is the address of the procedure's argument(s), and
@code{out} is the address of where to place the result(s); @code{inproc}
is used to encode the procedure's parameters, and @code{outproc} is used
to decode the procedure's results.  This routine returns zero if it
succeeds, or the value of @code{enum clnt_stat} cast to an integer if
it fails.  The routine
@ifinfo
@ref{clnt_perrno()}
@end ifinfo
@iftex
@code{clnt_perrno()}
@end iftex
is handy for translating failure statuses into messages. 

@strong{Warning:} calling remote procedures with this routine uses
@code{UDP/IP} as a transport; @pxref{clntudp_create()} for restrictions.
You do not have control of timeouts or authentication using this
routine.

@need 3200
@node clnt_broadcast()
@subsection clnt_broadcast()
@findex clnt_broadcast()

@example
enum clnt_stat
clnt_broadcast(prognum, versnum, procnum, inproc, in, outproc, out, eachresult)
u_long prognum, versnum, procnum;
char *in, *out;
xdrproc_t inproc, outproc;
resultproc_t eachresult;
@end example

@noindent Like
@ifinfo
@ref{callrpc()},
@end ifinfo
@iftex
@code{callrpc()},
@end iftex
except the call message is broadcast to all locally connected broadcast
nets. Each time it receives a response, this routine calls
@code{eachresult()}, whose form is:

@example
int
eachresult(out, addr)
char *out;
struct sockaddr_in *addr;
@end example

@noindent where @code{out} is the same as @code{out} passed to
@code{clnt_broadcast()}, except that the remote procedure's output is
decoded there; @code{addr} points to the address of the machine that
sent the results.  If @code{eachresult()} returns zero,
@code{clnt_broadcast()} waits for more replies; otherwise it returns
with appropriate status.

@strong{Warning:} broadcast sockets are limited in size to the maximum
transfer unit of the data link. For ethernet, this value is 1500 bytes.

@need 2600
@node clnt_call()
@subsection clnt_call()
@findex clnt_call()

@example
enum clnt_stat
clnt_call(clnt, procnum, inproc, in, outproc, out, tout)
CLIENT *clnt;
u_long procnum;
xdrproc_t inproc, outproc;
char *in, *out;
struct timeval tout;
@end example

@noindent A macro that calls the remote procedure @code{procnum}
 associated with the client handle, @code{clnt}, which is obtained with
an @code{RPC} client creation routine such as
@ifinfo
@ref{clnt_create()}.
@end ifinfo
@iftex
@code{clnt_create()}.
@end iftex
The parameter @code{in} is the address of the procedure's argument(s),
and @code{out} is the address of where to place the result(s);
@code{inproc} is used to encode the procedure's parameters, and
@code{outproc} is used to decode the procedure's results; @code{tout} is
the time allowed for results to come back.

@need 1400
@node clnt_destroy()
@subsection clnt_destroy()
@findex clnt_destroy()

@example
void
clnt_destroy(clnt)
CLIENT *clnt;
@end example

@noindent A macro that destroys the client's @code{RPC} handle.
Destruction usually involves deallocation of private data structures,
including @code{clnt} itself.  Use of @code{clnt} is undefined after
calling @code{clnt_destroy()}.  If the @code{RPC} library opened the
associated socket, it will close it also.  Otherwise, the socket remains
open.

@need 2000
@node clnt_create()
@subsection clnt_create()
@findex clnt_create()

@example
CLIENT *
clnt_create(host, prog, vers, proto)
char *host;
u_long prog, vers;
char *proto;
@end example

@noindent Generic client creation routine. @code{host} identifies 
the name of the remote host where the server is located.  @code{proto}
indicates which kind of transport protocol to use. The currently
supported values for this field are @code{"udp"} and @code{"tcp"}.
Default timeouts are set, but can be modified using
@ifinfo
@ref{clnt_control()}.
@end ifinfo
@iftex
@code{clnt_control()}.
@end iftex

@strong{Warning:} Using @code{UDP} has its shortcomings.  Since
@code{UDP}-based @code{RPC} messages can only hold up to 8 Kbytes of
encoded data, this transport cannot be used for procedures that take
large arguments or return huge results.

@need 2000
@node clnt_control()
@subsection clnt_control()
@findex clnt_control()

@example
bool_t
clnt_control(cl, req, info)
CLIENT *cl;
char *info;
@end example

@noindent A macro used to change or retrieve various information
about a client object.  @code{req} indicates the type of operation, and
@code{info} is a pointer to the information. For both @code{UDP} and
@code{TCP}, the supported values of @code{req} and their argument types
and what they do are:

@example
CLSET_TIMEOUT        struct timeval      @r{set total timeout}
CLGET_TIMEOUT        struct timeval      @r{get total timeout}
@end example

@strong{Note:} if you set the timeout using @code{clnt_control()}, the
timeout parameter passed to
@ifinfo
@ref{clnt_call()}
@end ifinfo
@iftex
@code{clnt_call()}
@end iftex
will be ignored in all future calls.

@example
CLGET_SERVER_ADDR    struct sockaddr_in  @r{get server's address}
@end example

The following operations are valid for @code{UDP} only:

@example
CLSET_RETRY_TIMEOUT  struct timeval      @r{set the retry timeout}
CLGET_RETRY_TIMEOUT  struct timeval      @r{get the retry timeout}
@end example

The retry timeout is the time that @code{UDP RPC} waits for the server
to reply before retransmitting the request.

@need 2000
@node clnt_freeres()
@subsection clnt_freeres()
@findex clnt_freeres()

@example
bool_t
clnt_freeres(clnt, outproc, out)
CLIENT *clnt;
xdrproc_t outproc;
char *out;
@end example

@noindent A macro that frees any data allocated by the @code{RPC/XDR}
system when it decoded the results of an @code{RPC} call.  The parameter
@code{out} is the address of the results, and @code{outproc} is the
@code{XDR} routine describing the results.  This routine returns one if
the results were successfully freed, and zero otherwise.

@need 1200
@node clnt_geterr()
@subsection clnt_geterr()
@findex clnt_geterr()

@example
void
clnt_geterr(clnt, errp)
CLIENT *clnt;
struct rpc_err *errp;
@end example

@noindent A macro that copies the error structure out of the client
handle to the structure at address @code{errp}.

@need 1600
@node clnt_pcreateerror()
@subsection clnt_pcreateerror()
@findex clnt_pcreateerror()

@example
void
clnt_pcreateerror(s)
char *s;
@end example

@noindent Print a message to standard error indicating
why a client @code{RPC} handle could not be created.  The message is
prepended with string @code{s} and a colon.  Used when a
@ifinfo
@ref{clnt_create()},
@end ifinfo
@iftex
@code{clnt_create()},
@end iftex
@ifinfo
@ref{clntraw_create()},
@end ifinfo
@iftex
@code{clntraw_create()},
@end iftex
@ifinfo
@ref{clnttcp_create()},
@end ifinfo
@iftex
@code{clnttcp_create()},
@end iftex
or
@ifinfo
@ref{clntudp_create()}
@end ifinfo
@iftex
@code{clntudp_create()}
@end iftex
call fails.

@need 1600
@node clnt_perrno()
@subsection clnt_perrno()
@findex clnt_perrno()

@example
void
clnt_perrno(stat)
enum clnt_stat stat;
@end example

@noindent Print a message to standard error corresponding to the 
condition indicated by @code{stat}.  Used after
@ifinfo
@ref{callrpc()}.
@end ifinfo
@iftex
@code{callrpc()}.
@end iftex

@need 1600
@node clnt_perror()
@subsection clnt_perror()
@findex clnt_perror()

@example
void 
clnt_perror(clnt, s)
CLIENT *clnt;
char *s;
@end example

@noindent Print a message to standard error indicating why an
@code{RPC} call failed; @code{clnt} is the handle used to do the call.
The message is prepended with string @code{s} and a colon.  Used after
@ifinfo
@ref{clnt_call()}.
@end ifinfo
@iftex
@code{clnt_call()}.
@end iftex

@need 1800
@node clnt_spcreateerror()
@subsection clnt_spcreateerror()
@findex clnt_spcreateerror()

@example
char *
clnt_spcreateerror(s)
char *s;
@end example

@noindent Like
@ifinfo
@ref{clnt_pcreateerror()},
@end ifinfo
@iftex
@code{clnt_pcreateerror()},
@end iftex
except that it returns a string instead of printing to the standard
error. 

@strong{Bugs:} returns pointer to static data that is overwritten on each
call.

@need 1800
@node clnt_sperrno()
@subsection clnt_sperrno()
@findex clnt_sperrno()

@example
char *
clnt_sperrno(stat)
enum clnt_stat stat;
@end example

@noindent Take the same arguments as
@ifinfo
@ref{clnt_perrno()},
@end ifinfo
@iftex
@code{clnt_perrno()},
@end iftex
but instead of sending a message to the standard error indicating why an
@code{RPC} call failed, return a pointer to a string which contains the
message.  The string ends with a @code{'\n'}.  @code{clnt_sperrno()} is
used instead of
@ifinfo
@ref{clnt_perrno()}
@end ifinfo
@iftex
@code{clnt_perrno()}
@end iftex
if the program does not have a standard error (as a program running as a
server quite likely does not), or if the programmer does not want the
message to be output with @code{printf}, or if a message format
different than that supported by
@ifinfo
@ref{clnt_perrno()}
@end ifinfo
@iftex
@code{clnt_perrno()}
@end iftex
is to be used.

@strong{Note:} unlike
@ifinfo
@ref{clnt_sperror()}
@end ifinfo
@iftex
@code{clnt_sperror()}
@end iftex
and
@ifinfo
@ref{clnt_spcreateerror()},
@end ifinfo
@iftex
@code{clnt_spcreateerror()},
@end iftex
@code{clnt_sperrno()} returns pointer to static data, but the result
will not get overwritten on each call.

@need 1400
@node clnt_sperror()
@subsection clnt_sperror()
@findex clnt_sperror()

@example
char *
clnt_sperror(rpch, s)
CLIENT *rpch;
char *s;
@end example

@noindent Like
@ifinfo
@ref{clnt_perror()},
@end ifinfo
@iftex
@code{clnt_perror()},
@end iftex
except that (like
@ifinfo
@ref{clnt_sperrno()})
@end ifinfo
@iftex
@code{clnt_sperrno()})
@end iftex
it returns a string instead of printing to standard error.

@strong{Bugs:} returns pointer to static data that is overwritten on
each call.

@need 2000
@node clntraw_create()
@subsection clntraw_create()
@findex clntraw_create()

@example
CLIENT *
clntraw_create(prognum, versnum)
u_long prognum, versnum;
@end example

@noindent This routine creates a toy @code{RPC} client for the
 remote program @code{prognum}, version @code{versnum}.  The transport
used to pass messages to the service is actually a buffer within the
process's address space, so the corresponding @code{RPC} server should
live in the same address space; see
@ifinfo
@ref{svcraw_create()}.
@end ifinfo
@iftex
@code{svcraw_create()}.
@end iftex
This allows simulation of @code{RPC} and acquisition of @code{RPC}
overheads, such as round trip times, without any kernel
interference. This routine returns @code{NULL} if it fails.

@need 3000
@node clnttcp_create()
@subsection clnttcp_create()
@findex clnttcp_create()

@example
CLIENT *
clnttcp_create(addr, prognum, versnum, sockp, sendsz, recvsz)
struct sockaddr_in *addr;
u_long prognum, versnum;
int *sockp;
u_int sendsz, recvsz;
@end example

@noindent This routine creates an @code{RPC} client for the remote
 program @code{prognum}, version @code{versnum}; the client uses
@code{TCP/IP} as a transport. The remote program is located at Internet
address @code{*addr}.  If @code{addr->sin_port} is zero, then it is set
to the actual port that the remote program is listening on (the remote
@code{portmap} service is consulted for this information). The parameter
@code{sockp} is a socket; if it is @code{RPC_ANYSOCK}, then this routine
opens a new one and sets @code{sockp}.  Since @code{TCP}-based
@code{RPC} uses buffered @code{I/O}, the user may specify the size of
the send and receive buffers with the parameters @code{sendsz} and
@code{recvsz}; values of zero choose suitable defaults.  This routine
returns @code{NULL} if it fails.

@need 3000
@node clntudp_create()
@subsection clntudp_create()
@findex clntudp_create()

@example
CLIENT *
clntudp_create(addr, prognum, versnum, wait, sockp)
struct sockaddr_in *addr;
u_long prognum, versnum;
struct timeval wait;
int *sockp;
@end example

@noindent This routine creates an @code{RPC} client for the remote
program @code{prognum}, version @code{versnum}; the client uses use
@code{UDP/IP} as a transport. The remote program is located at Internet
address @code{addr}.  If @code{addr->sin_port} is zero, then it is set
to actual port that the remote program is listening on (the remote
@code{portmap} service is consulted for this information). The parameter
@code{sockp} is a socket; if it is @code{RPC_ANYSOCK}, then this routine
opens a new one and sets @code{sockp}.  The @code{UDP} transport resends
the call message in intervals of @code{wait} time until a response is
received or until the call times out.  The total time for the call to
time out is specified by
@ifinfo
@ref{clnt_call()}.
@end ifinfo
@iftex
@code{clnt_call()}.
@end iftex

@strong{Warning:} since @code{UDP}-based @code{RPC} messages can only
hold up to 8 Kbytes of encoded data, this transport cannot be used for
procedures that take large arguments or return huge results.

@need 1600
@node clntudp_bufcreate()
@subsection clntudp_bufcreate()
@findex clntudp_bufcreate()

@example
CLIENT *
clntudp_bufcreate(addr, prognum, versnum, wait, sockp, sendsize, recosize)
struct sockaddr_in *addr;
u_long prognum, versnum;
struct timeval wait;
int *sockp;
unsigned int sendsize;
unsigned int recosize;
@end example

@noindent This routine creates an @code{RPC} client for the remote
program @code{prognum}, on @code{versnum}; the client uses use
@code{UDP/IP} as a transport. The remote program is located at Internet
address @code{addr}.  If @code{addr->sin_port} is zero, then it is set
to actual port that the remote program is listening on (the remote
@code{portmap} service is consulted for this information). The parameter
@code{sockp} is a socket; if it is @code{RPC_ANYSOCK}, then this routine
opens a new one and sets @code{sockp}.  The @code{UDP} transport resends
the call message in intervals of @code{wait} time until a response is
received or until the call times out.  The total time for the call to
time out is specified by
@ifinfo
@ref{clnt_call()}.
@end ifinfo
@iftex
@code{clnt_call()}.
@end iftex
This allows the user to specify the maximun packet size for sending and
receiving @code{UDP-based} @code{RPC} messages.

@need 1400
@node get_myaddress()
@subsection get_myaddress()
@findex get_myaddress()

@example
void
get_myaddress(addr)
struct sockaddr_in *addr;
@end example

@noindent Stuff the machine's @code{IP} address into @code{*addr}, 
without consulting the library routines that deal with host database or
name service.  The port number is always set to @code{htons(PMAPPORT)}.

@need 2000
@node pmap_getmaps()
@subsection pmap_getmaps()
@findex pmap_getmaps()

@example
struct pmaplist *
pmap_getmaps(addr)
struct sockaddr_in *addr;
@end example

@noindent A user interface to the @code{portmap} service, which returns
a list of the current @code{RPC} program-to-port mappings on the host
located at @code{IP} address @code{*addr}.  This routine can return
@code{NULL}.  The command
@ifinfo
@ref{portmap and rpcinfo, portmap}
@end ifinfo
@iftex
@code{portmap}
@end iftex
uses this routine.

@need 2400
@node pmap_getport()
@subsection pmap_getport()
@findex pmap_getport()

@example
u_short
pmap_getport(addr, prognum, versnum, protocol)
struct sockaddr_in *addr;
u_long prognum, versnum, protocol;
@end example

@noindent A user interface to the @code{portmap} service, which
 returns the port number on which waits a service that supports program
number @code{prognum}, version @code{versnum}, and speaks the transport
protocol associated with @code{protocol}.  The value of @code{protocol}
is most likely @code{IPPROTO_UDP} or @code{IPPROTO_TCP}.  A return value
of zero means that the mapping does not exist or that the @code{RPC}
system failured to contact the remote @code{portmap} service.  In the
latter case, the global variable
@ifinfo
@ref{rpc_createerr}
@end ifinfo
@iftex
@code{rpc_createerr}
@end iftex
contains the @code{RPC} status.

@need 3000
@node pmap_rmtcall()
@subsection pmap_rmtcall()
@findex pmap_rmtcall()

@example
enum clnt_stat
pmap_rmtcall(addr, prognum, versnum, procnum, 
             inproc, in, outproc, out, tout, portp)
struct sockaddr_in *addr;
u_long prognum, versnum, procnum;
char *in, *out;
xdrproc_t inproc, outproc;
struct timeval tout;
u_long *portp;
@end example

@noindent A user interface to the @code{portmap} service, which instructs
@code{portmap} on the host at @code{IP} address @code{*addr} to make an
@code{RPC} call on your behalf to a procedure on that host.  The
parameter @code{*portp} will be modified to the program's port number if
the procedure succeeds. The definitions of other parameters are
discussed in
@ifinfo
@ref{callrpc()}
@end ifinfo
@iftex
@code{callrpc()}
@end iftex
and
@ifinfo
@ref{clnt_call()}.
@end ifinfo
@iftex
@code{clnt_call()}.
@end iftex
This procedure should be used for a "ping" and nothing else.  See also
@ifinfo
@ref{clnt_broadcast()}.
@end ifinfo
@iftex
@code{clnt_broadcast()}.
@end iftex

@need 1800
@node pmap_set()
@subsection pmap_set()
@findex pmap_set()

@example
bool_t
pmap_set(prognum, versnum, protocol, port)
u_long prognum, versnum, protocol;
u_short port;
@end example

@noindent A user interface to the @code{portmap} service, which
establishes a mapping between the triple [prognum,versnum,protocol] and
@code{port} on the machine's @code{portmap} service. The value of
@code{protocol} is most likely @code{IPPROTO_UDP} or @code{IPPROTO_TCP}.
This routine returns one if it succeeds, zero otherwise.  Automatically
done by
@ifinfo
@ref{svc_register()}.
@end ifinfo
@iftex
@code{svc_register()}.
@end iftex

@need 1400
@node pmap_unset()
@subsection pmap_unset()
@findex pmap_unset()

@example
bool_t
pmap_unset(prognum, versnum)
u_long prognum, versnum;
@end example

@noindent A user interface to the @code{portmap} service, which destroys
all mapping between the triple [prognum,versnum,*] and @code{ports} on
the machine's @code{portmap} service. This routine returns one if it
succeeds, zero otherwise.

@need 3000
@node registerrpc()
@subsection registerrpc()
@findex registerrpc()

@example
int
registerrpc(prognum, versnum, procnum, procname, inproc, outproc)
u_long prognum, versnum, procnum;
char *(*procname) () ;
xdrproc_t inproc, outproc;
@end example

@noindent Register procedure @code{procname} with the
@code{RPC} service package.  If a request arrives for program
@code{prognum}, version @code{versnum}, and procedure @code{procnum},
@code{procname} is called with a pointer to its parameter(s);
@code{progname} should return a pointer to its static result(s);
@code{inproc} is used to decode the parameters while @code{outproc} is
used to encode the results.  This routine returns zero if the
registration succeeded, -1 otherwise. 

@strong{Warning:} remote procedures registered in this form are accessed
using the @code{UDP/IP} transport; see
@ifinfo
@ref{svcudp_create()}
@end ifinfo
@iftex
@code{svcudp_create()}
@end iftex
for restrictions.

@need 1000
@node rpc_createerr
@subsection rpc_createerr
@findex rpc_createerr

@example
struct rpc_createerr     rpc_createerr;
@end example

@noindent A global variable whose value is set by any @code{RPC}
client creation routine that does not succeed.  Use the routine
@ifinfo
@ref{clnt_pcreateerror()}
@end ifinfo
@iftex
@code{clnt_pcreateerror()}
@end iftex
to print the reason why.

@need 1400
@node svc_destroy()
@subsection svc_destroy()
@findex svc_destroy()

@example
void
svc_destroy(xprt)
SVCXPRT *xprt;
@end example

@noindent A macro that destroys the @code{RPC} service transport handle,
@code{xprt}.  Destruction usually involves deallocation of private data
structures, including @code{xprt} itself.  Use of @code{xprt} is
undefined after calling this routine.

@need 1600
@node svc_fdset
@subsection svc_fdset
@findex svc_fdset

@example
fd_set svc_fdset;
@end example

@noindent A global variable reflecting the @code{RPC} service side's
read file descriptor bit mask; it is suitable as a parameter to the
@code{select} system call. This is only of interest if a service
implementor does not call
@ifinfo
@ref{svc_run()},
@end ifinfo
@iftex
@code{svc_run()},
@end iftex
but rather does his own asynchronous event processing.  This variable is
read-only (do not pass its address to @code{select}!), yet it may change
after calls to
@ifinfo
@ref{svc_getreqset()}
@end ifinfo
@iftex
@code{svc_getreqset()}
@end iftex
or any creation routines.

@need 1200
@node svc_fds
@subsection svc_fds
@findex svc_fds

@example
int svc_fds;
@end example

@noindent Similar to 
@ifinfo 
@ref{svc_fdset},
@end ifinfo
@iftex
@code{svc_fdset},
@end iftex
but limited to 32 descriptors. This interface is obsoleted by
@ifinfo
@ref{svc_fdset}.
@end ifinfo
@iftex
@code{svc_fdset}.
@end iftex

@need 1800
@node svc_freeargs()
@subsection svc_freeargs()
@findex svc_freeargs()

@example
bool_t
svc_freeargs(xprt, inproc, in)
SVCXPRT *xprt;
xdrproc_t inproc;
char *in;
@end example

@noindent A macro that frees any data allocated by the @code{RPC/XDR}
system when it decoded the arguments to a service procedure using
@ifinfo
@ref{svc_getargs()}.
@end ifinfo
@iftex
@code{svc_getargs()}.
@end iftex
This routine returns 1 if the results were successfully freed, and zero
otherwise.

@need 2000
@node svc_getargs()
@subsection svc_getargs()
@findex svc_getargs()

@example
bool_t
svc_getargs(xprt, inproc, in)
SVCXPRT *xprt;
xdrproc_t inproc;
char *in;
@end example

@noindent A macro that decodes the arguments of an @code{RPC} request
associated with the @code{RPC} service transport handle, @code{xprt}.
The parameter @code{in} is the address where the arguments will be
placed; @code{inproc} is the @code{XDR} routine used to decode the
arguments.  This routine returns one if decoding succeeds, and zero
otherwise.

@need 1800
@node svc_getcaller()
@subsection svc_getcaller()
@findex svc_getcaller()

@example
struct sockaddr_in *
svc_getcaller(xprt)
SVCXPRT *xprt;
@end example

@noindent The approved way of getting the network address of the caller
of a procedure associated with the @code{RPC} service transport handle,
@code{xprt}.

@need 1800
@node svc_getreqset()
@subsection svc_getreqset()
@findex svc_getreqset()

@example
void
svc_getreqset(rdfds)
fd_set *rdfds;
@end example

@noindent This routine is only of interest if a service implementor
does not call
@ifinfo
@ref{svc_run()},
@end ifinfo
@iftex
@code{svc_run()},
@end iftex
but instead implements custom asynchronous event processing.  It is
called when the @code{select()} system call has determined that an
@code{RPC} request has arrived on some @code{RPC} @code{socket(s)};
@code{rdfds} is the resultant read file descriptor bit mask.  The
routine returns when all sockets associated with the value of
@code{rdfds} have been serviced.

@need 1200
@node svc_getreq()
@subsection svc_getreq()
@findex svc_getreq()

@example
void 
svc_getreq(rdfds)
int rdfds;
@end example

@noindent Similar to
@ifinfo
@ref{svc_getreqset()},
@end ifinfo
@iftex
@code{svc_getreqset()},
@end iftex
but limited to 32 descriptors. This interface is obsoleted by
@ifinfo
@ref{svc_getreqset()}.
@end ifinfo
@iftex
@code{svc_getreqset()}.
@end iftex

@need 3400
@node svc_register()
@subsection svc_register()
@findex svc_register()

@example
bool_t
svc_register(xprt, prognum, versnum, dispatch, protocol)
SVCXPRT *xprt;
u_long prognum, versnum;
void (*dispatch) ();
u_long protocol;
@end example

@noindent Associates @code{prognum} and @code{versnum} with the service
 dispatch procedure, @code{dispatch}.  If @code{protocol} is zero, the
service is not registered with the @code{portmap} service.  If
@code{protocol} is non-zero, then a mapping of the triple
[prognum,versnum,protocol] to @code{xprt->xp_port} is established with
the local @code{portmap} service (generally @code{protocol} is zero,
@code{IPPROTO_UDP} or @code{IPPROTO_TCP}).  The procedure
@code{dispatch} has the following form:

@example
void
dispatch(request, xprt)
struct svc_req *request;
SVCXPRT *xprt;
@end example

@noindent The @code{svc_register()} routine returns one if it succeeds,
and zero otherwise.

@need 1200
@node svc_run()
@subsection svc_run()
@findex svc_run()

@example
void
svc_run()
@end example

@noindent This routine never returns. It waits for @code{RPC}
requests to arrive, and calls the appropriate service procedure using
@ifinfo
@ref{svc_getreqset()}
@end ifinfo
@iftex
@code{svc_getreqset()}
@end iftex
when one arrives. This procedure is usually waiting for a
@code{select()} system call to return.

@need 1800
@node svc_sendreply()
@subsection svc_sendreply()
@findex svc_sendreply()

@example
bool_t
svc_sendreply(xprt, outproc, out)
SVCXPRT *xprt;
xdrproc_t outproc;
char *out;
@end example

@noindent Called by an @code{RPC} service's dispatch routine to send
the results of a remote procedure call.  The parameter @code{xprt} is
the request's associated transport handle; @code{outproc} is the
@code{XDR} routine which is used to encode the results; and @code{out}
is the address of the results.  This routine returns one if it succeeds,
zero otherwise.

@need 1400
@node svc_unregister()
@subsection svc_unregister()
@findex svc_unregister()

@example
void
svc_unregister(prognum, versnum)
u_long prognum, versnum;
@end example

@noindent Remove all mapping of the double [prognum,versnum] to 
dispatch routines, and of the triple [prognum,versnum,*] to port number.

@need 1800
@node svcerr_auth()
@subsection svcerr_auth()
@findex svcerr_auth()

@example
void
svcerr_auth(xprt, why)
SVCXPRT *xprt;
enum auth_stat why;
@end example

@noindent Called by a service dispatch routine that refuses to perform
a remote procedure call due to an authentication error.

@need 1400
@node svcerr_decode()
@subsection svcerr_decode()
@findex svcerr_decode()

@example
void
svcerr_decode(xprt)
SVCXPRT *xprt;
@end example

@noindent Called by a service dispatch routine that cannot successfully
decode its parameters. Also @xref{svc_getargs()}.

@need 1400
@node svcerr_noproc()
@subsection svcerr_noproc()
@findex svcerr_noproc()

@example
void
svcerr_noproc(xprt)
SVCXPRT *xprt;
@end example

@noindent Called by a service dispatch routine that does not implement
the procedure number that the caller requests.

@need 1400
@node svcerr_noprog()
@subsection svcerr_noprog()
@findex svcerr_noprog()

@example
void
svcerr_noprog(xprt)
SVCXPRT *xprt;
@end example

@noindent Called when the desired program is not registered with the
@code{RPC} package. Service implementors usually do not need this
routine.

@need 1400
@node svcerr_progvers()
@subsection svcerr_progvers()
@findex svcerr_progvers()

@example
void
svcerr_progvers(xprt)
SVCXPRT *xprt;
@end example

@noindent Called when the desired version of a program is not registered
with the @code{RPC} package. Service implementors usually do not need
this routine.

@need 1400
@node svcerr_systemerr()
@subsection svcerr_systemerr()
@findex svcerr_systemerr()

@example
void
svcerr_systemerr(xprt)
SVCXPRT *xprt;
@end example

@noindent Called by a service dispatch routine when it detects a system
error not covered by any particular protocol.  For example, if a service
can no longer allocate storage, it may call this routine.

@need 1600
@node svcerr_weakauth()
@subsection svcerr_weakauth()
@findex svcerr_weakauth()

@example
void
svcerr_weakauth(xprt)
SVCXPRT *xprt;
@end example

@noindent Called by a service dispatch routine that refuses to perform
a remote procedure call due to insufficient authentication parameters.
The routine calls @code{svcerr_auth(xprt, AUTH_TOOWEAK)}.

@need 2200
@node svcraw_create()
@subsection svcraw_create()
@findex svcraw_create()

@example
SVCXPRT *
svcraw_create()
@end example

@noindent This routine creates a toy @code{RPC} service transport,
to which it returns a pointer.  The transport is really a buffer within
the process's address space, so the corresponding @code{RPC} client
should live in the same address space; see
@ifinfo
@ref{clntraw_create()}.
@end ifinfo
@iftex
@code{clntraw_create()}.
@end iftex
This routine allows simulation of @code{RPC} and acquisition of
@code{RPC} overheads (such as round trip times), without any kernel
interference.  This routine returns @code{NULL} if it fails.

@need 2200
@node svctcp_create()
@subsection svctcp_create()
@findex svctcp_create()

@example
SVCXPRT *
svctcp_create(sock, send_buf_size, recv_buf_size)
int sock;
u_int send_buf_size, recv_buf_size;
@end example

@noindent This routine creates a @code{TCP/IP}-based @code{RPC}
service transport, to which it returns a pointer.  The transport is
associated with the socket @code{sock}, which may be @code{RPC_ANYSOCK},
in which case a new socket is created.  If the socket is not bound to a
local @code{TCP} port, then this routine binds it to an arbitrary port.
Upon completion, @code{xprt->xp_sock} is the transport's socket
descriptor, and @code{xprt->xp_port} is the transport's port number.
This routine returns @code{NULL} if it fails. Since @code{TCP}-based
@code{RPC} uses buffered @code{I/O}, users may specify the size of
buffers; values of zero choose suitable defaults.

@need 2200
@node svcfd_create()
@subsection svcfd_create()
@findex svcfd_create()

@example
SVCXPRT *
svcfd_create(fd, sendsize, recvsize)
int fd;
u_int sendsize;
u_int recvsize;
@end example

@noindent Create a service on top of any open descriptor. Typically,
this descriptor is a connected socket for a stream protocol such as
@code{TCP.}  @code{sendsize} and @code{recvsize} indicate sizes for the
send and receive buffers.  If they are zero, a reasonable default is
chosen.

@need 2000
@node svcudp_bufcreate()
@subsection svcudp_bufcreate()
@findex svcudp_bufcreate()

@example
SVCXPRT *
svcudp_bufcreate(sock, sendsize, recvsize)
int sock;
u_int sendsize, recvsize;
@end example

@noindent This routine creates a @code{UDP/IP}-based @code{RPC}
service transport, to which it returns a pointer.  The transport is
associated with the socket @code{sock}, which may be @code{RPC_ANYSOCK},
in which case a new socket is created.  If the socket is not bound to a
local @code{UDP} port, then this routine binds it to an arbitrary
port. Upon completion, @code{xprt->xp_sock} is the transport's socket
descriptor, and @code{xprt->xp_port} is the transport's port number.
This routine returns @code{NULL} if it fails.  This allows the user to
specify the maximun packet size for sending and receiving
@code{UDP}-based @code{RPC} messages.

@need 2000
@node svcudp_create()
@subsection svcudp_create()
@findex svcudp_create()

@example
SVCXPRT *
svcudp_create(sock)
int sock;
@end example

@noindent This routine creates a @code{UDP/IP}-based @code{RPC}
service transport, to which it returns a pointer.  This is done by
calling the
@ifinfo
@pxref{svcudp_bufcreate()}
@end ifinfo
@iftex
@code{svcudp_bufcreate()}
@end iftex
with the default buffer size @code{UDPMSGSIZE}.

@need 1400
@node xdr_accepted_reply()
@subsection xdr_accepted_reply()
@findex xdr_accepted_reply()

@example
bool_t XDRFUN
xdr_accepted_reply(xdrs, ar)
XDR *xdrs;
struct accepted_reply *ar;
@end example

@noindent Used for encoding @code{RPC}
reply messages. This routine is useful for users who wish to generate
RPC-style messages without using the @code{RPC} package.

@need 1400
@node xdr_authunix_parms()
@subsection xdr_authunix_parms()
@findex xdr_authunix_parms()

@example
bool_t XDRFUN
xdr_authunix_parms(xdrs, aupp)
XDR *xdrs;
struct authunix_parms *aupp;
@end example

@noindent Used for describing @code{UNIX} credentials. This routine
is useful for users who wish to generate these credentials without using
the @code{RPC} authentication package.

@need 1400
@node xdr_callhdr()
@subsection xdr_callhdr()
@findex xdr_callhdr()

@example
void
xdr_callhdr(xdrs, chdr)
XDR *xdrs;
struct rpc_msg *chdr;
@end example

@noindent Used for describing @code{RPC} call header messages.
This routine is useful for users who wish to generate @code{RPC-style}
messages without using the @code{RPC} package.

@need 1400
@node xdr_callmsg()
@subsection xdr_callmsg()
@findex xdr_callmsg()

@example
bool_t XDRFUN
xdr_callmsg(xdrs, cmsg)
XDR *xdrs;
struct rpc_msg *cmsg;
@end example

@noindent Used for describing @code{RPC} call messages.
This routine is useful for users who wish to generate @code{RPC-style}
messages without using the @code{RPC} package.

@need 1400
@node xdr_opaque_auth()
@subsection xdr_opaque_auth()
@findex xdr_opaque_auth()

@example
bool_t XDRFUN
xdr_opaque_auth(xdrs, ap)
XDR *xdrs;
struct opaque_auth *ap;
@end example

@noindent Used for describing @code{RPC} authentication information messages.
This routine is useful for users who wish to generate @code{RPC-style}
messages without using the @code{RPC} package.

@need 1400
@node xdr_pmap()
@subsection xdr_pmap()
@findex xdr_pmap()

@example
bool_t XDRFUN
xdr_pmap(xdrs, regs)
XDR *xdrs;
struct pmap *regs;
@end example

@noindent Used for describing parameters to various @code{portmap}
procedures, externally.  This routine is useful for users who wish to
generate these parameters without using the @code{pmap} interface.

@need 1400
@node xdr_pmaplist()
@subsection xdr_pmaplist()
@findex xdr_pmaplist()

@example
bool_t XDRFUN
xdr_pmaplist(xdrs, rp)
XDR *xdrs;
struct pmaplist **rp;
@end example

@noindent Used for describing a list of port mappings, externally.
This routine is useful for users who wish to generate these parameters
without using the @code{pmap} interface.

@need 1400
@node xdr_rejected_reply()
@subsection xdr_rejected_reply()
@findex xdr_rejected_reply()

@example
bool_t XDRFUN
xdr_rejected_reply(xdrs, rr)
XDR *xdrs;
struct rejected_reply *rr;
@end example

@noindent Used for describing @code{RPC} reply messages.
This routine is useful for users who wish to generate @code{RPC-style}
messages without using the @code{RPC} package.

@need 1600
@node xdr_replymsg()
@subsection xdr_replymsg()
@findex xdr_replymsg()

@example
bool_t XDRFUN
xdr_replymsg(xdrs, rmsg)
XDR *xdrs;
struct rpc_msg *rmsg;
@end example

@noindent Used for describing @code{RPC} reply messages.
This routine is useful for users who wish to generate @code{RPC} style
messages without using the @code{RPC} package.

@need 1600
@node xprt_register()
@subsection xprt_register()
@findex xprt_register()

@example
void
xprt_register(xprt)
SVCXPRT *xprt;
@end example

@noindent After @code{RPC} service transport handles are created,
they should register themselves with the @code{RPC} service package.
This routine modifies the global variable
@ifinfo
@ref{svc_fds}.
@end ifinfo
@iftex
@code{svc_fds}.
@end iftex
Service implementors usually do not need this routine.

@need 1600
@node xprt_unregister()
@subsection xprt_unregister()
@findex xprt_unregister()

@example
void
xprt_unregister(xprt)
SVCXPRT *xprt;
@end example

@noindent Before an @code{RPC} service transport handle is destroyed,
it should unregister itself with the @code{RPC} service package.  This
routine modifies the global variable
@ifinfo
@ref{svc_fds}.
@end ifinfo
@iftex
@code{svc_fds}.
@end iftex
Service implementors usually do not need this routine.

